// ==============================================================
//
//  Copyright (c) 2002-2003 Alex Vinokur.
//
//  ------------------------------------------------------------
//  This file is part of C++ Simulator of a Post Machine.
//
//  C++ Simulator of a Post Machine is free software; 
//  you can redistribute it and/or modify it
//  under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License,
//  or (at your option) any later version.
//
//  C++ Simulator of a Post Machine is distributed in the hope 
//  that it will be useful, but WITHOUT ANY WARRANTY; 
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with C++ Simulator of a Post Machine; 
//  if not, write to the Free Software Foundation, Inc., 
//  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//  ------------------------------------------------------------
// 
//  mailto:alexvn@connect.to
//  http://up.to/alexv
//
// ==============================================================




// ##############################################################
//
//  SOFTWARE : Nondeterministic Post Machine (C++ Simulator)
//  FILE     : commands.h
//
//  DESCRIPTION :
//         Class CommandAction (Definition)
//
// ##############################################################




// ###############
#ifndef _COMMANDS_H
#define _COMMANDS_H

#include "defs.h"



// =========
#define TYPE_NAME_MAX_LEN	64
typedef char name64_t[TYPE_NAME_MAX_LEN];

enum CommandType
{
  UNDEF_CommandType,
  SEQUENTIAL_cmd,
  CONDITIONAL_JUMP_cmd,
  END_cmd,
  LAST_CommandType
};

enum OperationType
{
  UNDEF_OperationType,
  STAY_ON_op,
  LEFT_SHIFT_op,
  RIGHT_SHIFT_op,
  WRITE_LABEL_op,
  ERASE_LABEL_op,
  CHECK_IF_MARKED_op,
  CHECK_IF_BLANK_op,
  STOP_op,
  LAST_OperationType
};


// =========
class CommandAction
{
friend class NondeterministicPostMachine;
  private :
    vector<OperationType>  cur_operations_;
    vector<command_t>      next_commands_; 	 // size = 0, 1, 2
    static vector<string>  op_names_s;
    

  public :
    CommandAction ();
    CommandAction (
		const vector<OperationType>&  cur_operations_i,
		const vector<command_t>&      next_commands_i
		);
    ~CommandAction ();	

   CommandType          get_command_type () const;
   static CommandType   get_command_type_S (OperationType op_i);
   static OperationType get_op_type_S (const string& op_name_i);
   static bool          is_allowed_op_name_S (const string& op_name_i);
   static size_t        get_op_names_max_size_S ();
   static string        get_allowed_op_names_S (CommandType cmd_type = UNDEF_CommandType);
   static string        get_cmd_type_name_S (CommandType cmd_type = UNDEF_CommandType);

   vector<command_t>    get_next_commands () const;	

   string		getstr_cur_operation_names (size_t size_i = 0) const;
   string		getstr_next_commands (size_t size_i = 0) const;


};


#endif	// _COMMANDS_H
