// ==============================================================
//
//  Copyright (c) 2002-2003 Alex Vinokur.
//
//  ------------------------------------------------------------
//  This file is part of C++ Simulator of a Post Machine.
//
//  C++ Simulator of a Post Machine is free software; 
//  you can redistribute it and/or modify it
//  under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License,
//  or (at your option) any later version.
//
//  C++ Simulator of a Post Machine is distributed in the hope 
//  that it will be useful, but WITHOUT ANY WARRANTY; 
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with C++ Simulator of a Post Machine; 
//  if not, write to the Free Software Foundation, Inc., 
//  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//  ------------------------------------------------------------
// 
//  mailto:alexvn@connect.to
//  http://up.to/alexv
//
// ==============================================================




// ##############################################################
//
//  SOFTWARE : Nondeterministic Post Machine (C++ Simulator)
//  FILE     : run.cpp
//
//  DESCRIPTION :
//         Class Run (Implementation)
//
// ##############################################################




// ###############
#include "run.h"

#define	setw3	3
#define	setw4	4
#define	setw5	5
#define	setw7	7
#define	setw8	8
#define	setw9	9
#define	setw10	10
#define	setw12	12
#define	setw13	13
#define	setw14	14
#define	setw15	15
#define	setw23	23
#define	setw43	43

const string	space3 (3, ' ');
const string	space4 (4, ' ');
const string	space5 (5, ' ');
const string	space6 (6, ' ');
const string	space8 (8, ' ');
const string	space9 (9, ' ');

const string	prefixa ("%%");

#define INITIAL_COMMAND_HARD_NO 1

enum MetafileFields
{
  // --- DON'T CHANGE ---
  METAFILE_FIELD__DESCR_FILE_NAME,
  METAFILE_FIELD__NUMBER_OF_TAPES,
  METAFILE_FIELD__COMMANDS_FILE_NAME,
  METAFILE_FIELD__INPUT_WORDS_FILE1_NAME
};



// ---- Tape numeration : 0, 1, 2, ...
#define	COMMANDS_FILE__TOTAL_SEQ_CMD_FIELDS(number_of_tapes)       (number_of_tapes + 2)
#define	COMMANDS_FILE__TOTAL_COND_CMD_FIELDS(number_of_tapes)      (number_of_tapes + 3)
#define	COMMANDS_FILE__TOTAL_END_CMD_FIELDS                        2

#define	COMMANDS_FILE__CUR_COMMAND_FIELD                           0
#define	COMMANDS_FILE__CUR_OPERATION_FIELD_OF_NON_END_CMD(tape_no) (tape_no + 1)
#define	COMMANDS_FILE__NEXT_FIRST_COMMAND_FIELD(number_of_tapes)   (number_of_tapes + 1)


// =========
// =========
// Constructor-0
Aux::Aux ()
{
  init();
}

// =========
// Constructor-0
Aux::~Aux ()
{
}

// =========
void Aux::init ()
{
  assert (help_request_keys_.empty());
  help_request_keys_.push_back ("-h");
  help_request_keys_.push_back ("--help");
  help_request_keys_.push_back ("?");
}

// =========
// =========
string			Run::metafile_name_s;
vector<vector<string> >	Run::metafile_data_s;

Aux			aux;
const vector<string>	Run::help_request_keys_s (aux.help_request_keys_);


// =========
// =========
// Constructor-1
Run::Run (
		size_t			serial_number_i,
		size_t			number_of_tapes_i,
		const string&		descr_file_name_i,
		const string&		commands_file_name_i,
		const vector<string>&	input_words_files_names_i
		)
		:
		serial_number_ (serial_number_i),
		last_command_no_ (0)
{
ostringstream oss1, oss21, oss22;
  cout	<< endl;
  
  oss1	<< space8 
	<< prefixa
	<< string (setw5, '=') 
	<< " Nondeterministic Post Machine# "
	<< serial_number_ 
	<< " "
	<< string (setw4, '='); 
  
  oss21	<< space8 
	<< prefixa
	<< string (setw3, '=') 
	<< " PreProcessing Metafile Data : "; 
  
  oss22	<< " " 
	<< string (setw3, '='); 

  cout << space8 << prefixa << string (setw43, '=') << endl;
  cout << oss1.str() << endl;
  cout << oss21.str() << "BEGIN" << oss22.str() << endl;
  cout << space8 << prefixa << string (setw43, '=') << endl;

  check_results_ = init_number_of_tapes (number_of_tapes_i);
  if (check_results_) check_results_ = init_descr (descr_file_name_i);
  if (check_results_) check_results_ = init_commands (commands_file_name_i);
  if (check_results_) check_results_ = create_commands();
  if (check_results_) check_results_ = init_input_words (input_words_files_names_i);

  cout << space8 << prefixa << string (setw43, '=') << endl;
  cout << oss21.str() << " END " << oss22.str() << endl;
  cout << oss1.str() << endl;
  cout << space8 << prefixa << string (setw43, '=') << endl;

  if (check_results_)
  {
    check_results_ = invoke_machine ();
  }

  if (!check_results_)
  {
    FATAL_MSG	("Can't invoke this Nondeterministic Post Machine");
  }
} 

// =========
// Destructor
Run::~Run ()
{
} 


// =========
bool Run::init_number_of_tapes (size_t number_of_tapes_i)
{
  number_of_tapes_ = number_of_tapes_i;
  if (number_of_tapes_ == 0)
  {
    FATAL_MSG	("Illegal number of tapes : "
		<< number_of_tapes_
		);
    return false;
  }
  return true;
}


// =========
bool Run::init_descr(const string& descr_file_name_i)
{
bool	ret_bool_value;

  descr_file_name_ = descr_file_name_i;
  ret_bool_value = read_descr_file ();

  if (!ret_bool_value)
  {
    cout << endl;
    show_descr_file_structure_S ();
    show_descr_file_sample_S ();
    cout << endl;
  }
  return ret_bool_value; 

}


// =========
bool Run::init_commands(const string& commands_file_name_i)
{

bool	ret_bool_value;

  commands_file_name_ = commands_file_name_i;
  ret_bool_value = read_commands_file ();

  if (!ret_bool_value)
  {
    cout << endl;
    show_commands_file_structure ();
    show_commands_file_sample_S ();
    cout << endl;
  }

  return ret_bool_value; 

}

// =========
bool Run::init_input_words (const vector<string>& input_words_files_names_i)
{

  for (size_t i = 0; i < input_words_files_names_i.size(); i++)
  {
    input_words_files_names_.push_back (input_words_files_names_i[i]);
  }

  assert (!input_words_files_names_.empty());
  for (size_t i = 0; i < input_words_files_names_.size(); i++)
  {
    if (!read_input_words_file (input_words_files_names_[i]))
    {
      cout << endl;
      show_input_words_file_structure_S ();
      show_input_words_file_sample_S ();
      cout << endl;
      return false;
    }
  }

  show_input_words_files_content ();

  return true; 
}


// =========
bool Run::create_commands()
{
command_t             cur_command;
vector<OperationType> cur_operations;
vector<command_t>     next_commands;

typedef Commands_t::value_type value_type;
pair<Commands_t::iterator, bool> the_pair;

  commands_map_.clear();

  assert (!commands_file_data_.empty());

  for (size_t i = 0; i < commands_file_data_.size(); i++)
  {
    const size_t actual_line_no = i + 1;
    cur_operations.clear();
    next_commands.clear();

    bool cur_cmd_is_initial = false;
    bool cur_cmd_is_halting = false;
    bool cur_cmd_is_internal = false;

    const bool rc = parse_command (
			actual_line_no, 
			commands_file_data_[i], 
			cur_command, 
			cur_operations,
			next_commands,
			cur_cmd_is_initial,
			cur_cmd_is_halting,
			cur_cmd_is_internal
			);
    if (!rc)
    {
      ostringstream oss;
      copy (commands_file_data_[i].begin(), commands_file_data_[i].end(), ostream_iterator<string> (oss, " "));

      FATAL_MSG (  "Command line#" 
                << actual_line_no
                << " is invalid" 
                << "; ===> command line : "
                << oss.str()
                << endl
                );
      return false;

    }
    assert (rc);

    if (commands_map_.count(cur_command) == 0) commands_map_[cur_command] = vector<CommandAction> ();
    assert (commands_map_.count(cur_command) == 1);

    commands_map_[cur_command].push_back (CommandAction (cur_operations, next_commands));

    if (cur_cmd_is_initial)  initial_command_numbers_.push_back(cur_command);
    if (cur_cmd_is_halting)  halting_command_numbers_.push_back(cur_command);
    if (cur_cmd_is_internal) internal_command_numbers_.push_back(cur_command);

  }

  // -----------------------
  if (halting_command_numbers_.empty())
  {
    FATAL_MSG (  "Missing halting command" 
	      << endl
              );

    return false;
  }


  return true;
}


// =========
bool Run::parse_command (
			size_t                  line_no_i,
			const vector<string>&   cmd_i,
			command_t&              cur_command_no_o,
			vector<OperationType>&  cur_operations_o,
			vector<command_t>&      next_command_no_o,
			bool&			cur_cmd_is_initial_o,
			bool&			cur_cmd_is_halting_o,
			bool&			cur_cmd_is_internal_o
			)
{
  cur_operations_o.clear();
  next_command_no_o.clear();
  assert (number_of_tapes_ > 0);

  cur_cmd_is_initial_o = false;
  cur_cmd_is_halting_o = false;
  cur_cmd_is_internal_o = false;
  
  if (cmd_i.empty())
  {
    FATAL_MSG ("Empty command line#" << line_no_i);
    return false;
  }

ostringstream oss;
  copy (cmd_i.begin(), cmd_i.end(), ostream_iterator<string> (oss, " "));
  oss << endl;
const string str_cmd_line (oss.str());

  // -----------------------------
  // ---- check cur command_no ---
  if (!stream_cast (cmd_i[COMMANDS_FILE__CUR_COMMAND_FIELD], cur_command_no_o))
  {
    FATAL_MSG (  "Problematic cur_command_no in command line#" 
              << line_no_i 
              << "; command_no = <" 
              << cmd_i[COMMANDS_FILE__CUR_COMMAND_FIELD] 
              << ">"
              << "; ===> command line : "
              << str_cmd_line
              );
    return false;
  }

const bool check_cmd_no = (cur_command_no_o >= INITIAL_COMMAND_HARD_NO) && ((cur_command_no_o - last_command_no_) <= 1);
  if (!check_cmd_no)
  {
    FATAL_MSG (  "Problematic cur_command_no in command line#" 
              << line_no_i
              << "; command_no = " 
              << cur_command_no_o
              << ", prev_command_no = "
              << last_command_no_
              << "; Note! command_no must be = "
              << last_command_no_
              << " or "
              << (last_command_no_ + 1)
              << "; ===> command line : "
              << str_cmd_line
              );
    return false;
  }
  last_command_no_ = cur_command_no_o;


  // ------------------------------
  // ---- check operation types ---

vector<CommandType>   cmd_types;

  assert (cur_operations_o.empty());
  for (size_t k = 1; k < (number_of_tapes_ + 1); k++)
  {
    const string cur_op_name = cmd_i[k];
    if (!CommandAction::is_allowed_op_name_S(cur_op_name))
    {
      FATAL_MSG (  "Illegal operation type <"
                << cur_op_name
                << "> in command line#" 
                << line_no_i 
                << ", operation = " 
                << cur_op_name
                << "; Allowed operations : "
                << CommandAction::get_allowed_op_names_S()
                << "; ===> command line : "
                << str_cmd_line
                );
      return false;
    }

    {
      size_t must_be_fields = 0;
      switch (CommandAction::get_command_type_S(CommandAction::get_op_type_S (cur_op_name)))
      {
        case END_cmd :
          must_be_fields = COMMANDS_FILE__TOTAL_END_CMD_FIELDS;
          break;

        case SEQUENTIAL_cmd :
          must_be_fields = COMMANDS_FILE__TOTAL_SEQ_CMD_FIELDS(number_of_tapes_);
          break;

        case CONDITIONAL_JUMP_cmd :
          must_be_fields = COMMANDS_FILE__TOTAL_COND_CMD_FIELDS(number_of_tapes_);
          break;

        default :
          assert (0);
          break;   // unused

      } // switch
      assert (must_be_fields >= COMMANDS_FILE__TOTAL_END_CMD_FIELDS);

      if (cmd_i.size() != must_be_fields)
      {
        FATAL_MSG (  "Illegal number of fields ("
		  << cmd_i.size()
                  << ")"
                  << " in command line#" 
                  << line_no_i 
                  << ", must be "
		  << must_be_fields
                  << " fields"
                  << "; ===> command line : "
                  << str_cmd_line
                  );
        return false;
      }
    }

    // -------------------------------
    cur_operations_o.push_back (CommandAction::get_op_type_S (cur_op_name));
    assert (cur_operations_o.back() > UNDEF_OperationType);
    assert (cur_operations_o.back() < LAST_OperationType);

    cmd_types.push_back (CommandAction::get_command_type_S (cur_operations_o.back()));
    assert (cmd_types.back() > UNDEF_CommandType);
    assert (cmd_types.back() < LAST_CommandType);

    // -----------------
    if (cur_operations_o.back() == STOP_op) break;
  }
  assert (!cur_operations_o.empty());
  assert (!cmd_types.empty());
  assert (cur_operations_o.size() == cmd_types.size());

  // -------------------------------------
  for (size_t k = (number_of_tapes_ + 1); k < cmd_i.size(); k++)
  {
    size_t next_command_no;
    if (!stream_cast (cmd_i[k], next_command_no))
    {
      FATAL_MSG (  "Problematic next_command_no in command line#" 
                << line_no_i 
                << "; command_no = <" 
                << cmd_i[k] 
                << ">"
                << "; ===> command line : "
                << str_cmd_line
                );
      return false;
    }

    if (!((next_command_no >= 1) && (next_command_no <= commands_file_data_.size())))
    {
      FATAL_MSG (  "Problematic next_command_no in command line#" 
                << line_no_i
                << "; next_command_no = " 
                << next_command_no
                << "; Note! command_no must belong to [1-"
                << commands_file_data_.size()
                << "]"
                << "; ===> command line : "
                << str_cmd_line
                );
      return false;
    }

    assert ((next_command_no >= 1) && (next_command_no <= commands_file_data_.size()));
    next_command_no_o.push_back(next_command_no);
  }


  // -----------------------
  for (size_t k = 0; k < cmd_types.size(); k++)
  {
    if (cmd_types[k] != cmd_types.front())
    {
      FATAL_MSG (  "Command line#" 
                << line_no_i 
                << " contains operations of different types"
                << "; ===> command line : "
                << str_cmd_line
                );

      return false;
    }
  }

  // --------------------
  assert (number_of_tapes_ > 0);


size_t normative_total_fields = 0; 
  switch (cmd_types.front())
  {
    case END_cmd :
      normative_total_fields = COMMANDS_FILE__TOTAL_END_CMD_FIELDS;
      cur_cmd_is_halting_o = true;
      break;

    case SEQUENTIAL_cmd :
      normative_total_fields = COMMANDS_FILE__TOTAL_SEQ_CMD_FIELDS(number_of_tapes_);
      cur_cmd_is_internal_o = true;
      break;

    case CONDITIONAL_JUMP_cmd :
      normative_total_fields = COMMANDS_FILE__TOTAL_COND_CMD_FIELDS(number_of_tapes_);
      cur_cmd_is_internal_o = true;
      break;

    default :
      assert (0);
      break;   // unused

  } // switch
  assert (normative_total_fields > 0); 
  if (cur_command_no_o == INITIAL_COMMAND_HARD_NO) 
  {
    cur_cmd_is_initial_o  = true;
    cur_cmd_is_internal_o = false;
  }

  assert (cur_cmd_is_initial_o || cur_cmd_is_halting_o || cur_cmd_is_internal_o);
  assert (!(cur_cmd_is_initial_o && cur_cmd_is_internal_o));
  assert (!(cur_cmd_is_halting_o && cur_cmd_is_internal_o));

  // -----------
  if (cmd_i.size() != normative_total_fields)
  {
    FATAL_MSG (  "Illegal number of items in command line#" 
              << line_no_i 
              << " : actual = " 
              << cmd_i.size()
              << ", must be = "
              << normative_total_fields
              << "; ===> command line : "
              << str_cmd_line
              );
    return false;
  }
  assert (cmd_i.size() == normative_total_fields);

  switch (cmd_types.front())
  {
    case END_cmd :
      assert (next_command_no_o.empty());
      break;

    case SEQUENTIAL_cmd :
      assert (next_command_no_o.size() == 1);
      break;

    case CONDITIONAL_JUMP_cmd :
      assert (next_command_no_o.size() == 2);
      break;

    default :
      assert (0);
      break;   // unused

  } // switch


  // -------------------------
  return true;
}


// =========
bool Run::invoke_machine ()
{
ostringstream oss1, oss21, oss22;
  cout	<< endl;
  
  oss1	<< space8 
	<< prefixa
	<< string (setw5, '=') 
	<< " Nondeterministic Post Machine# "
	<< serial_number_ 
	<< " "
	<< string (setw4, '='); 
  
  oss21	<< space8 
	<< prefixa
	<< string (setw7, '=') 
	<< " Machine Definition : "; 
  
  oss22	<< " " 
	<< string (setw8, '='); 

  cout << endl;
  cout << space8 << prefixa << string (setw43, '=') << endl;
  cout << oss1.str() << endl;  
  cout << oss21.str() << "BEGIN" << oss22.str() << endl;
  cout << space8 << prefixa << string (setw43, '=') << endl;

  assert (initial_command_numbers_.size() == 1);
  assert (halting_command_numbers_.size() >= 1);

NondeterministicPostMachine post_machine (
	number_of_tapes_,
	descr_file_data_,
	initial_command_numbers_,
	halting_command_numbers_,
	internal_command_numbers_,
	commands_map_
	);


  if (!post_machine.get_check_results())
  {
    FATAL_MSG	("Invalid Nondeterministic Post Machine definition");
  }

  cout << space8 << prefixa << string (setw43, '=') << endl;
  cout << oss21.str() << " END " << oss22.str() << endl;
  cout << oss1.str() << endl;  
  cout << space8 << prefixa << string (setw43, '=') << endl;

  if (!post_machine.get_check_results())	return false;

  assert (post_machine.get_check_results());

  // ------------------------
  for (size_t i = 0; i < input_words_files_data_.size(); i++)
  {
    ostringstream oss31, oss32, oss41, oss42;
    oss31	<< space8 
		<< prefixa
		<< string (setw9, '=') 
		<< " Processing Input Words ";

    oss41	<< space8 
		<< prefixa
		<< string (setw13, '=') 
		<< " Set# "
		<< (i + 1)
		<< " : "; 
  
    oss32	<< " " 
    		<< string (setw9, '='); 

    oss42	<< " " 
		<< string (setw14, '='); 

    cout << endl;
    cout << endl;
    cout << endl;
    cout << space8 << prefixa << string (setw43, '=') << endl;
    cout << oss1.str() << endl;  
    cout << oss31.str() << oss32.str() << endl;
    cout << oss41.str() << "BEGIN" << oss42.str() << endl;
    cout << space8 << prefixa << string (setw43, '=') << endl;

    post_machine.process_input (input_words_files_data_[i]);

    cout << endl;
    cout << space8 << prefixa << string (setw43, '=') << endl;
    cout << oss41.str() << " END " << oss42.str() << endl;
    cout << oss31.str() << oss32.str() << endl;
    cout << oss1.str() << endl;  
    cout << space8 << prefixa << string (setw43, '=') << endl;

  }
  return true;
} // invoke_machine


// =========
bool Run::read_file_S (const string& filename_i, vector<vector<string> >& data_o)
{
string file_line;

  assert (!filename_i.empty());

  ifstream fin (filename_i.c_str());
  if (!fin)
  {
    DIAGNOSTIC_MSG	("Unable to open file <"
			<< filename_i
			<< 
			">"
			);
    return false;
  }


  cout << endl;
  cout << "\t" << string (setw23, '-') << endl;
  cout << "\t--- File " << filename_i << endl;
  cout << "\t" << string (setw23, '-') << endl;

  data_o.clear();
  for (int i = 0; getline (fin, file_line); i++)
  {
    data_o.push_back(vector<string>());
    istringstream isstr (file_line.c_str ());

    copy(
         istream_iterator<string>(isstr),
         istream_iterator<string> (),
         back_inserter (data_o[i])
        );

    cout << "\t  Row#" << setw(3) << (i + 1) << " ---> ";
    copy (
          data_o[i].begin (),
          data_o[i].end (),
          ostream_iterator<string> (cout, " ")
         );
    cout << endl;


  } // while
  cout << "\t" << string (setw23, '-') << endl;
  cout << endl;

  return true;

} // read_file_S


// =========
bool Run::read_metafile_S (const string& filename_i)
{
// ------------------------------------------
// Metafile structure :
// Field#1 : <description file name>
// Field#2 : <number of tapes>
// Field#5 : <commands file name>
// Field#6 : <input words file#1 name>
// Field#7 : <input words file#2 name> [optionaly]
// .........
// Field#n : <input words file#(n-4) name> [optionaly]
// ------------------------------------------
#define MIN_NUMBER_OF_FIELDS_IN_META_FILE_ROW	4

  metafile_name_s = filename_i;
  if (!read_file_S (metafile_name_s , metafile_data_s))
  {
    FATAL_MSG	("Unable to read metafile <"
		<< metafile_name_s
		<< ">"
		);
    assert (metafile_data_s.empty());
    return false;
  }

  if (metafile_data_s.empty())
  {
    FATAL_MSG	("Metafile <"
		<< metafile_name_s
		<< "> is empty"
		);
    metafile_data_s.clear();
    return false;
  }

  for (size_t i = 0; i< metafile_data_s.size(); i++)
  {
    if (metafile_data_s[i].empty())
    {
      FATAL_MSG	("Metafile <"
		<< metafile_name_s
		<< ">, Row#"
		<< (i + 1)
		<< " is empty"
		);
      metafile_data_s.clear();
      return false;
    }

    if (metafile_data_s[i].size() < MIN_NUMBER_OF_FIELDS_IN_META_FILE_ROW)
    {
      FATAL_MSG	("Metafile <"
		<< metafile_name_s
		<< ">, Row#"
		<< (i + 1)
		<< " ---> illegal number of fields : "
		<< metafile_data_s[i].size()
		<< "; MIN ALLOWED = "
		<< MIN_NUMBER_OF_FIELDS_IN_META_FILE_ROW
		);
      metafile_data_s.clear();
      return false;
    }

    // --- Checking ---
    for (size_t j = 0; j < metafile_data_s[i].size(); j++)
    {

      assert (!metafile_data_s[i][j].empty());

      switch (j)
      {
        case METAFILE_FIELD__NUMBER_OF_TAPES :
          if (!(is_udec (const_cast<char*>(metafile_data_s[i][j].c_str()))))
          {
            FATAL_MSG	("Metafile <"
			<< metafile_name_s
			<< ">, Row#"
			<< (j + 1)
			<< " ---> non-numeric Number-Of-Tapes: <"
			<< metafile_data_s[i][j]
			<< ">"
			);
            metafile_data_s.clear();
            return false;
          }
          break;

        case METAFILE_FIELD__DESCR_FILE_NAME :
        case METAFILE_FIELD__COMMANDS_FILE_NAME :
          // Do nothing
          break;

        default :
          // Do nothing
          break;

       } // switch (j)

    } // for (size j = 0; ...

  } // for (size_t i = 0; ...
  // ----------------
  show_metafile_content_S ();
  return true;

} // read_metafile_S




// =========
bool Run::read_descr_file ()
{
  assert (!descr_file_name_.empty());
  if (!read_file_S (descr_file_name_, descr_file_data_))
  {
    WARNING_MSG	("Unable to read descr file <"
		<< descr_file_name_
		<< ">"
		);
    assert (descr_file_data_.empty());
    return true;
  }

  if (descr_file_data_.empty())
  {
    WARNING_MSG	("Description file <"
		<< descr_file_name_
		<< "> is empty"
		);
    descr_file_data_.clear();
    return true;
  }

  // ----------------
  show_descr_file_content ();

  return true;

} // read_descr_file



// =========
bool Run::read_commands_file ()
{
  assert (!commands_file_name_.empty());
  if (!read_file_S (commands_file_name_, commands_file_data_))
  {
    FATAL_MSG	("Unable to read commands file <"
		<< commands_file_name_ 
		<< ">"
		);
    assert (commands_file_data_.empty());
    return false;
  }


  if (commands_file_data_.empty())
  {
    FATAL_MSG	("Commands file <"
		<< commands_file_name_
		<< "> is empty"
		);
    commands_file_data_.clear();
    return false;
  }

  for (size_t i = 0; i < commands_file_data_.size(); i++)
  {
    if (commands_file_data_[i].empty())
    {
      FATAL_MSG	("Commands file <"
		<< commands_file_name_
		<< ">, Row#"
		<< (i + 1)
		<< " is empty"
		);
      commands_file_data_.clear();
      return false;
    }
  } // for (size_t i = 0; ...


  // ----------------
  show_commands_file_content ();


  return true;
} // read_commands_file

// =========
bool Run::read_input_words_file (const string& filename_i)
{
  input_words_files_data_.push_back (vector<vector<string> >());
  assert (!filename_i.empty());
  if (!read_file_S (filename_i, *input_words_files_data_.rbegin()))
  {
    FATAL_MSG	("Unable to read input words file <"
		<< filename_i
		<< ">"
		);
    assert (input_words_files_data_.rbegin()->empty());
    return false;
  }

  // ----------------
  // show_input_words_files_content ();

  return true;
} // read_input_words_file


// =========
void Run::show_metafile_structure_S ()
{
// ------------------------------------------
// Metafile structure :
// Field#1 : <descr file name>
// Field#2 : <number of tapes>
// Field#5 : <commands file name>
// Field#6 : <input words file#1 name>
// Field#7 : <input words file#2 name> [optionaly]
// .........
// Field#n : <input words file#(n-4) name> [optionaly]
// ------------------------------------------
const string text1 ("Metafile structure");

  cout	<< endl;
  cout	<< space3
	<< string (setw10, '#')
	<< " "
	<< text1
	<< " "
	<< string (setw10, '#')
	<< endl;

  cout	<< space4 << "Each row contains data related to some Nondeterministic Post Machine" << endl;
  cout	<< space3 << string (setw10 + 1 + text1.size() + 1 + setw10, '-') << endl;

  cout	<< space4 << "Field#1 : " << "description file name" << endl;
  cout	<< space4 << "Field#2 : " << "number of tapes" << endl;
  cout	<< space4 << "Field#4 : " << "commands file name" << endl;
  cout	<< space4 << "Field#6 : " << "input words file#1 name" << endl;
  cout	<< space4 << "Field#7 : " << "input words file#2 name [optional]" << endl;
  cout	<< space4 << "....... : " << "........." << endl;

  cout	<< space4 << "Field#n : " << "input words file#(n-4) name [optional]" << endl;

  cout	<< space3 << string (setw10 + 1 + text1.size() + 1 + setw10, '#') << endl;
} // show_metafile_structure_S



// =========
void Run::show_descr_file_structure_S  ()
{
const string text1 ("Description file structure");

  cout	<< endl;
  cout	<< space3
	<< string (setw10, '=')
	<< " "
	<< text1
	<< " "
	<< string (setw10, '=')
	<< endl;

  cout	<< space4 << "--- Your text :" << endl;
  cout	<< space4 << "--- Description of your Nondeterministic Post Machine" << endl;

  cout	<< space3 << string (setw10 + 1 + text1.size() + 1 + setw10, '=') << endl;

} // show_descr_file_structure_S




// =========
void Run::show_commands_file_structure () const
{
  show_commands_file_structure_S();

const string text1 ("Command file structure");

  cout	<< endl;
  cout	<< space3
	<< string (setw10, '=')
	<< " "
	<< text1
	<< " "
	<< string (setw10, '=')
	<< endl;

size_t	field_no = 0;

const string text_field ("Field#");

const string text2  ("command_no");
const string text31 ("sequential_operation");
const string text32 ("conditional_operation");
const string text33 ("end_operation");
const string text41 ("next_command_no");
const string text42 ("next_command_no (if true)");
const string text43 ("next_command_no (if false)");
size_t the_setw = 0;

  the_setw = MAX_VALUE (the_setw, text2.size());
  the_setw = MAX_VALUE (the_setw, text31.size());
  the_setw = MAX_VALUE (the_setw, text32.size());
  the_setw = MAX_VALUE (the_setw, text33.size());
  the_setw = MAX_VALUE (the_setw, text41.size());
  the_setw = MAX_VALUE (the_setw, text42.size());
  the_setw = MAX_VALUE (the_setw, text43.size());
  the_setw++; 

const size_t cur_setw = 5;

  cout << space4 << "Each row contains some command" << endl;
  cout << space4 << "Number of tapes = " << number_of_tapes_ << endl;

  // -------------------------
  field_no = 0;
  cout << endl;
  cout << space3 << string (setw10 + 1 + text1.size() + 1 + setw10, '-') << endl;

  cout << space4 << "::: Structure of " << CommandAction::get_cmd_type_name_S (SEQUENTIAL_cmd) << "command (" << CommandAction::get_allowed_op_names_S (SEQUENTIAL_cmd) << ") :::" << endl;

  cout << space4 << text_field << setw(cur_setw) << right << field_no++ << " : " << text2.c_str() << endl;
  for (size_t i = 0; i < number_of_tapes_; i++)
  {
    cout << space4 << text_field << setw(cur_setw) << right << field_no++ << " : " << setw (the_setw) << left << text31.c_str() << " on tape#" << i << endl;
  }
  cout << space4 << text_field << setw(cur_setw) << right << (number_of_tapes_ + 1) << " : " << setw (the_setw) << left << text41.c_str() << endl;

  // -------------------------
  field_no = 0;
  cout << endl;
  cout << space3 << string (setw10 + 1 + text1.size() + 1 + setw10, '-') << endl;

  cout << space4 << "::: Structure of " << CommandAction::get_cmd_type_name_S (CONDITIONAL_JUMP_cmd) << "command (" << CommandAction::get_allowed_op_names_S (CONDITIONAL_JUMP_cmd)  << ") :::" << endl;

  cout << space4 << text_field << setw(cur_setw) << right << field_no++ << " : " << text2.c_str() << endl;
  for (size_t i = 0; i < number_of_tapes_; i++)
  {
    cout << space4 << text_field << setw(cur_setw) << right << field_no++ << " : " << setw (the_setw) << left << text32.c_str() << " on tape#" << i << endl;
  }
  cout << space4 << text_field << setw(cur_setw) << right << (number_of_tapes_ + 1) << " : " << setw (the_setw) << left << text42.c_str() << endl;
  cout << space4 << text_field << setw(cur_setw) << right << (number_of_tapes_ + 2) << " : " << setw (the_setw) << left << text43.c_str() << endl;

  // -------------------------
  field_no = 0;
  cout << endl;
  cout << space3 << string (setw10 + 1 + text1.size() + 1 + setw10, '-') << endl;

  cout << space4 << "::: Structure of " << CommandAction::get_cmd_type_name_S (END_cmd) << "command (" << CommandAction::get_allowed_op_names_S (END_cmd) << ") :::" << endl;

  cout << space4 << text_field << setw(cur_setw) << right << field_no++ << " : " << text2.c_str() << endl;
  cout << space4 << text_field << setw(cur_setw) << right << field_no << " : " << setw (the_setw) << left << text33.c_str() << endl;


  cout	<< space3 << string (setw10 + 1 + text1.size() + 1 + setw10, '=') << endl;
  cout << endl;

} // show_commands_file_structure



// =========
void Run::show_commands_file_structure_S()
{
const string text1 ("Command file structure");

  cout	<< endl;
  cout	<< space3
	<< string (setw10, '=')
	<< " "
	<< text1
	<< " "
	<< string (setw10, '=')
	<< endl;

size_t	field_no = 0;
const string field_n ("n");
const string field_n1 ("n+1");
const string field_n2 ("n+2");

const string text_field ("Field#");

const string text2  ("command_no");
const string text31 ("sequential_operation");
const string text32 ("conditional_operation");
const string text33 ("end_operation");
const string text41 ("next_command_no");
const string text42 ("next_command_no (if true)");
const string text43 ("next_command_no (if false)");
size_t the_setw = 0;

  the_setw = MAX_VALUE (the_setw, text2.size());
  the_setw = MAX_VALUE (the_setw, text31.size());
  the_setw = MAX_VALUE (the_setw, text32.size());
  the_setw = MAX_VALUE (the_setw, text33.size());
  the_setw = MAX_VALUE (the_setw, text41.size());
  the_setw = MAX_VALUE (the_setw, text42.size());
  the_setw = MAX_VALUE (the_setw, text43.size());
  the_setw++; 

const size_t tmp_no   = 3;
const size_t cur_setw = 5;


  cout << space4 << "Each row contains some command" << endl;
  cout << space4 << "Let n be number of tapes" << endl;

  // -------------------------
  field_no = 0;
  cout << endl;
  cout << space3 << string (setw10 + 1 + text1.size() + 1 + setw10, '-') << endl;

  cout << space4 << "::: Structure of " << CommandAction::get_cmd_type_name_S (SEQUENTIAL_cmd) << "command (" << CommandAction::get_allowed_op_names_S (SEQUENTIAL_cmd) << ") :::" << endl;

  cout << space4 << text_field << setw(cur_setw) << right << field_no++ << " : " << text2.c_str() << endl;
  for (size_t i = 0; i < tmp_no; i++)
  {
    cout << space4 << text_field << setw(cur_setw) << right << field_no++ << " : " << setw (the_setw) << left << text31.c_str() << " on tape#" << i << endl;
  }
  cout << space4 << string (25, '.') << endl;
  cout << space4 << text_field << setw(cur_setw) << right << field_n << " : " << setw (the_setw) << left << text31.c_str() << " on tape#" << "n" << endl;
  cout << space4 << text_field << setw(cur_setw) << right << field_n1 << " : " << setw (the_setw) << left << text41.c_str() << endl;

  // -------------------------
  field_no = 0;
  cout << endl;
  cout << space3 << string (setw10 + 1 + text1.size() + 1 + setw10, '-') << endl;

  cout << space4 << "::: Structure of " << CommandAction::get_cmd_type_name_S (CONDITIONAL_JUMP_cmd) << "command (" << CommandAction::get_allowed_op_names_S (CONDITIONAL_JUMP_cmd)  << ") :::" << endl;

  cout << space4 << text_field << setw(cur_setw) << right << field_no++ << " : " << text2.c_str() << endl;
  for (size_t i = 0; i < tmp_no; i++)
  {
    cout << space4 << text_field << setw(cur_setw) << right << field_no++ << " : " << setw (the_setw) << left << text32.c_str() << " on tape#" << i << endl;
  }
  cout << space4 << string (25, '.') << endl;
  cout << space4 << text_field << setw(cur_setw) << right << field_n << " : " << setw (the_setw) << left << text32.c_str() << " on tape#" << "n" << endl;
  cout << space4 << text_field << setw(cur_setw) << right << field_n1 << " : " << setw (the_setw) << left << text42.c_str() << endl;
  cout << space4 << text_field << setw(cur_setw) << right << field_n2 << " : " << setw (the_setw) << left << text43.c_str() << endl;

  // -------------------------
  field_no = 0;
  cout << endl;
  cout << space3 << string (setw10 + 1 + text1.size() + 1 + setw10, '-') << endl;

  cout << space4 << "::: Structure of " << CommandAction::get_cmd_type_name_S (END_cmd) << "command (" << CommandAction::get_allowed_op_names_S (END_cmd) << ") :::" << endl;

  cout << space4 << text_field << setw(cur_setw) << right << field_no++ << " : " << text2.c_str() << endl;
  cout << space4 << text_field << setw(cur_setw) << right << field_no << " : " << setw (the_setw) << left << text33.c_str() << endl;


  cout	<< space3 << string (setw10 + 1 + text1.size() + 1 + setw10, '=') << endl;
  cout << endl;

} // show_commands_file_structure_S


			     
// =========
void Run::show_input_words_file_structure_S ()
{
const string text1 ("Input words file structure");

  cout	<< endl;
  cout	<< space3
	<< string (setw10, '=')
	<< " "
	<< text1
	<< " "
	<< string (setw10, '=')
	<< endl;

size_t	row_no = 0;
const string last_row_no ("n-1");

const string text_row ("Row#");

size_t the_setw = 0;

const string text2 ("input word");
  the_setw = MAX_VALUE (the_setw, text2.size());
  assert (the_setw != 0);

size_t	tmp_no = 3;
const size_t	the2_setw = last_row_no.size() + 2;


  cout	<< space4 << "Each row contains input word for some tape" << endl;
  cout	<< space4 << "Let n be number of tapes" << endl;
  cout	<< space3 << string (setw10 + 1 + text1.size() + 1 + setw10, '-') << endl;

  for (size_t i = 0; i < tmp_no; i++)
  {
    cout << space4 << text_row << setw(the2_setw) << right << row_no++ << " : " << setw (the_setw) << left << text2.c_str() << " on tape#" << i << endl;
  }
  cout << space4 << string (20, '.') << endl;

  cout << space4 << text_row << setw(the2_setw) << right << last_row_no.c_str() << " : " << setw (the_setw) << left << text2.c_str() << " on tape#" << "(n-1)" << endl;

  cout	<< space3 << string (setw10 + 1 + text1.size() + 1 + setw10, '=') << endl;


  cout  << space3
        << "Note! Symbol " 
        << "\"" 
        << HEAD_START_POSITION_POINTER_REDERVED_SYMBOL 
        << "\"" 
        << " may be used as (optional) pointer"
        << endl

        << space3
        << "      to start position of machine head on tape(s)."
        << endl

        << space3
        << "      In this case that symbol should be inserted"
        << endl

        << space3
        << "      before a pointed position"
        << endl;


} // show_input_words_file_structure_S


// =========
void Run::show_metafile_sample_S ()
{
const string text1("Metafile sample");
vector<vector<string> >	sample;

  sample.push_back(vector<string>());
  sample.rbegin()->push_back("sample1.dsc");
  sample.rbegin()->push_back("1");
  sample.rbegin()->push_back("sample1.cmd");
  sample.rbegin()->push_back("sample11.in");
  sample.rbegin()->push_back("sample12.in");
  sample.rbegin()->push_back("sample13.in");

  sample.push_back(vector<string>());
  sample.rbegin()->push_back("sample2.dsc");
  sample.rbegin()->push_back("3");
  sample.rbegin()->push_back("sample2.cmd");
  sample.rbegin()->push_back("sample21.in");


  cout	<< endl;
  cout	<< space3
	<< string (setw10, '#')
	<< " "
	<< text1
	<< " "
	<< string (setw10, '#')
	<< endl;

  for (size_t i = 0; i < sample.size(); i++)
  {
    copy (sample[i].begin(), sample[i].end(), ostream_iterator<string> (cout, " "));
    cout << endl;
  }

  cout	<< space3 << string (setw10 + 1 + text1.size() + 1 + setw10, '#') << endl;

}


// =========
void Run::show_descr_file_sample_S  ()
{
const string text1("Description file sample");
vector<vector<string> >	sample;


  sample.push_back(vector<string>());
  sample.push_back(vector<string>());
  sample.rbegin()->push_back("An addition of one to a number");

  sample.push_back(vector<string>());
  sample.push_back(vector<string>());
  sample.rbegin()->push_back("Source : V.A.Uspensky, \"A Post Machine\" (in Russian),");


  sample.push_back(vector<string>());
  sample.rbegin()->push_back("         Moscow, \"Nauka\", 1979.");

  sample.push_back(vector<string>());
  sample.push_back(vector<string>());
  sample.push_back(vector<string>());
  sample.rbegin()->push_back("The program adds one to a number.");

  sample.push_back(vector<string>());
  sample.rbegin()->push_back("A number \'n\' is represented by (n+1) 1-s.");

  sample.push_back(vector<string>());
  sample.rbegin()->push_back("Sample :");

  sample.push_back(vector<string>());
  sample.rbegin()->push_back("  0 is represented as  1");

  sample.push_back(vector<string>());
  sample.rbegin()->push_back("  1 is represented as  1 1");

  sample.push_back(vector<string>());
  sample.rbegin()->push_back("  3 is represented as  1 1 1 1");

  sample.push_back(vector<string>());
  sample.push_back(vector<string>());
  sample.push_back(vector<string>());
  sample.rbegin()->push_back("Number of tapes used : 1");


  sample.push_back(vector<string>());
  sample.push_back(vector<string>());
  sample.rbegin()->push_back("Input  : A number \'n\'");

  sample.push_back(vector<string>());
  sample.rbegin()->push_back("Output : Resulting number \'n+1\'");

  sample.push_back(vector<string>());
  sample.push_back(vector<string>());
  sample.rbegin()->push_back("Sample-1");


  sample.push_back(vector<string>());
  sample.rbegin()->push_back("Input  : 1 1 1 1");

  sample.push_back(vector<string>());
  sample.rbegin()->push_back("Output : 1 1 1 1 1");

  sample.push_back(vector<string>());
  sample.push_back(vector<string>());
  sample.rbegin()->push_back("Sample-2");

  sample.push_back(vector<string>());
  sample.rbegin()->push_back("Input  : b b b 1 1 1 1");

  sample.push_back(vector<string>());
  sample.rbegin()->push_back("Output : 1 1 1 1 1 ");

  sample.push_back(vector<string>());

  cout	<< endl;
  cout	<< space3
	<< string (setw10, '=')
	<< " "
	<< text1
	<< " "
	<< string (setw10, '=')
	<< endl;

  for (size_t i = 0; i < sample.size(); i++)
  {
    copy (sample[i].begin(), sample[i].end(), ostream_iterator<string> (cout, " "));
    cout << endl;
  }

  cout	<< space3 << string (setw10 + 1 + text1.size() + 1 + setw10, '=') << endl;

}

// =========
void Run::show_commands_file_sample_S  ()
{
const string text1("Commands file sample");
const string text2("Number of tapes = 1");
vector<vector<string> >	sample;

typedef char trow_t [10];

const trow_t row01[] = {"1", " BLANK", " 6", " 2" }; 
const trow_t row02[] = {"2", " LEFT ", " 3" }; 
const trow_t row03[] = {"3", " BLANK", " 4", " 2"}; 
const trow_t row04[] = {"4", " WRITE", " 5" }; 
const trow_t row05[] = {"5", " STOP" }; 
const trow_t row06[] = {"6", " RIGHT", " 1" }; 


  sample.push_back(vector<string> (row01, row01 + sizeof (row01)/sizeof(*row01)));
  sample.push_back(vector<string> (row02, row02 + sizeof (row02)/sizeof(*row02)));
  sample.push_back(vector<string> (row03, row03 + sizeof (row03)/sizeof(*row03)));
  sample.push_back(vector<string> (row04, row04 + sizeof (row04)/sizeof(*row04)));
  sample.push_back(vector<string> (row05, row05 + sizeof (row05)/sizeof(*row05)));
  sample.push_back(vector<string> (row06, row06 + sizeof (row06)/sizeof(*row06)));

  cout	<< endl;
  cout	<< space3
	<< string (setw10, '=')
	<< " "
	<< text1
	<< " "
	<< string (setw10, '=')
	<< endl;

  cout	<< space3
	<< string (setw12, '-')
	<< " "
	<< text2
	<< " "
	<< string (setw12, '-')
	<< endl;

  for (size_t i = 0; i < sample.size(); i++)
  {
    copy (sample[i].begin(), sample[i].end(), ostream_iterator<string> (cout, " "));
    cout << endl;
  }

  cout	<< space3 << string (setw10 + 1 + text1.size() + 1 + setw10, '=') << endl;

}

// =========
void Run::show_input_words_file_sample_S  ()
{
// ------ sample-1 ------
const string text11("Input words file sample-1");
const string text12("Number of tapes = 1");
vector<vector<string> >	sample1;

typedef char irow_t [3];

const irow_t row101[] = {"1", "1", "1", "1" }; 

  sample1.push_back(vector<string> (row101, row101 + sizeof (row101)/sizeof(*row101)));

  cout	<< endl;
  cout	<< space3
	<< string (setw10, '=')
	<< " "
	<< text11
	<< " "
	<< string (setw10, '=')
	<< endl;
  cout	<< space3
	<< string (setw12, '-')
	<< " "
	<< text12
	<< " "
	<< string (setw12, '-')
	<< endl;

  for (size_t i = 0; i < sample1.size(); i++)
  {
    copy (sample1[i].begin(), sample1[i].end(), ostream_iterator<string> (cout, " "));
    cout << endl;
  }

  cout	<< space3 << string (setw10 + 1 + text11.size() + 1 + setw10, '=') << endl;


// ------ sample-2 ------
const string text21("Input words file sample-2");
const string text22("Number of tapes = 1");
vector<vector<string> >	sample2;

typedef char irow_t [3];

const irow_t row201[] = {"1", "1", HEAD_START_POSITION_POINTER_REDERVED_SYMBOL, "1", "1" }; 

  sample2.push_back(vector<string> (row201, row201 + sizeof (row201)/sizeof(*row201)));

  cout	<< endl;
  cout	<< space3
	<< string (setw10, '=')
	<< " "
	<< text21
	<< " "
	<< string (setw10, '=')
	<< endl;
  cout	<< space3
	<< string (setw12, '-')
	<< " "
	<< text22
	<< " "
	<< string (setw12, '-')
	<< endl;

  for (size_t i = 0; i < sample2.size(); i++)
  {
    copy (sample2[i].begin(), sample2[i].end(), ostream_iterator<string> (cout, " "));
    cout << endl;
  }

  cout	<< space3 << string (setw10 + 1 + text21.size() + 1 + setw10, '=') << endl;

  cout	<< space3
        << "Note! Symbol " 
        << "\"" 
        << HEAD_START_POSITION_POINTER_REDERVED_SYMBOL 
        << "\"" 
        << " is (optional) pointer to start position of machine head"
        << endl;


}


// =========
void Run::show_metafile_content_S ()
{
size_t	text_setw = 0;
const string text1 ("Number of tapes");
const string text4 ("Commands file name");
const string text5 ("Input words files names");
const string text6 ("Description file name");

  text_setw = MAX_VALUE (text_setw, text1.size());
  text_setw = MAX_VALUE (text_setw, text4.size());
  text_setw = MAX_VALUE (text_setw, text5.size());
  text_setw = MAX_VALUE (text_setw, text6.size());


  cout << space3 << string (setw10, '=') << " Data selected from Metafile " << metafile_name_s << " " << string (setw10, '=') << endl;

  for (size_t i = 0; i < metafile_data_s.size(); i++)
  {
    cout << space5 << string (setw10, '-') << " Nondeterministic Post Machine# " << (i + 1) << " " << string (setw10, '-') << endl;
    cout << space6 << setw(text_setw) << left << text6.c_str() << " : " << metafile_data_s[i][METAFILE_FIELD__DESCR_FILE_NAME] << endl;
    cout << space6 << setw(text_setw) << left << text1.c_str() << " : " << metafile_data_s[i][METAFILE_FIELD__NUMBER_OF_TAPES] << endl;
    cout << space6 << setw(text_setw) << left << text4.c_str() << " : " << metafile_data_s[i][METAFILE_FIELD__COMMANDS_FILE_NAME] << endl;
    cout << space6 << setw(text_setw) << left << text5.c_str() << " : ";
    copy (metafile_data_s[i].begin() + METAFILE_FIELD__INPUT_WORDS_FILE1_NAME, metafile_data_s[i].end(), ostream_iterator<string> (cout, " "));
    cout << endl;
    cout << endl;
  }


} // show_metafile_content_S


// =========
void Run::show_descr_file_content  () const
{

  for (size_t i = 0; i < descr_file_data_.size(); i++)
  {
    cout << space6;
    copy (descr_file_data_[i].begin(), descr_file_data_[i].end(), ostream_iterator<string> (cout, " "));
    cout << endl;
  }

  cout << endl;
				 
} // show_descr_file_content



// =========
void Run::show_commands_file_content  () const
{
  for (size_t i = 0; i < commands_file_data_.size(); i++)
  {
    //cout << space6 << "Row#" << setw(4) << right << i << " : ";
    cout << space6 << "Rule" << " :   ";
    copy (commands_file_data_[i].begin(), commands_file_data_[i].end(), ostream_iterator<string> (cout, " "));
    cout << endl;
  }

  cout << endl;

} // show_commands_file_content

// =========
void Run::show_input_words_files_content  () const
{
  for (size_t i = 0; i < input_words_files_data_.size(); i++)
  {
    cout << space6 << "Input words --> Set#" << setw(3) << (i + 1) << endl;
    for (size_t j = 0; j < input_words_files_data_[i].size(); j++)
    {
      cout << space9 << "Tape#" << setw(3) << j << " : ";
      copy (input_words_files_data_[i][j].begin(), input_words_files_data_[i][j].end(), ostream_iterator<symbol_t> (cout, " "));
      cout << endl;
    }
    cout << endl;
  }


} // show_input_words_files_content


// =========
void Run::invoke_post_machine_S (
		size_t		serial_number_i,
		size_t		number_of_tapes_i,
		const string&	descr_file_name_i,
		const string&	commands_file_name_i,
		const vector<string>&	input_words_files_names_i
		)
{
ostringstream oss1, oss2;

  cout	<< endl;
  cout	<< endl;
  cout	<< endl;
  cout	<< endl;
  
  oss1	<< space5 
	<< string (setw10, '#') 
	<< " Nondeterministic Post Machine# " 
	<< serial_number_i 
	<< " : "; 
  
  oss2	<< " " 
	<< string (setw10, '#'); 

  cout << oss1.str() << "BEGIN" << oss2.str() << endl;


Run run_inst(
	serial_number_i,
	number_of_tapes_i,
	descr_file_name_i,
	commands_file_name_i,
	input_words_files_names_i
	);

  if (!run_inst.check_results_)
  {
    cout	<< space5 
    		<< "---> " 
       		<< "Invalid Data !!!" 
      		<< endl; 
  }
  cout	<< endl;
  cout	<< endl;

  cout << oss1.str() << " END " << oss2.str() << endl;

} // invoke_post_machine_S


// =========
void Run::show_usage_S (const string& exe_name_i)
{
  cout << "\tUSAGE    : " 
       << exe_name_i 
       << " [ metafile-name : Default = " 
       << DEFAULT_METAFILE 
       << " ]"
       << endl;
 
  cout << "\t         : " 
       << exe_name_i 
       << " [ ";
  for (size_t i = 0; i < help_request_keys_s.size(); i++)
  {
    if (i) cout << " | ";
    cout << help_request_keys_s[i];
  }
  cout << " ]" << endl;

  cout << endl;
  cout << "\tEXAMPLES : " << exe_name_i << endl;
  cout << "\t         : " << exe_name_i << " metafile2" << endl;
  for (size_t i = 0; i < help_request_keys_s.size(); i++)
  {
    cout << "\t         : " << exe_name_i << " " << help_request_keys_s[i] << endl;
  }

}

// =========
void Run::show_help_S (const string& exe_name_i)
{
  show_usage_S(exe_name_i);

  cout << endl;
  show_metafile_structure_S ();
  cout << endl;
  show_descr_file_structure_S ();
  cout << endl;
  show_commands_file_structure_S ();
  cout << endl;
  show_input_words_file_structure_S ();
  cout << endl;

  show_sample_S();
}


// =========
void Run::show_sample_S ()
{
  cout << endl << endl;
  show_metafile_sample_S ();
  cout << endl;		    

  cout << endl;		    

const string prefix1 (9, ' ');	   
const string prefix2 (5, ' ');	   
const string l_border ("/// ---");	   
const string r_border ("--- \\\\\\");	   

const string text1 ("Sample of Post Machine Definition");
  cout << endl;
  cout << endl;
  cout << ""
       << prefix2 << l_border << " " << text1 << " : " << "BEGIN" << " " << r_border 
       << endl;

  cout << "" 
       << prefix1 << "An example of a Post Machine : An addition of one to a number"
       << endl
       << prefix1 << "* from \"A Post Machine\" [1979] (in Russian)"
       << endl
       << prefix1 << "* by V.A.Uspensky"
       << endl
       << prefix1 << "is used in samples below" 
       << endl;

  show_descr_file_sample_S ();
  cout << endl;
  show_commands_file_sample_S ();
  cout << endl;
  show_input_words_file_sample_S ();
  cout << endl;

  cout << ""
       << prefix2 << l_border << " " << text1 << " : " << "END" << " " << r_border 
       << endl;
  cout << endl;

}


// =========
void Run::run (uint argc, char** argv)
{
  cout << endl;
  cout << endl;
  cout << "\t---> YOUR COMMAND LINE : ";
  for (uint i = 0; i < argc; i++)
  {
    cout << argv[i] << " ";
  }
  cout << endl;
  cout << endl;

  assert (!help_request_keys_s.empty());

  if (argc >= 2)
  {
    if (find (help_request_keys_s.begin(), help_request_keys_s.end(), string(argv[1])) != help_request_keys_s.end())
    {
      show_help_S(argv[0]);
      return;
    }
  }

  if (argc == 1)
  {
    cout << endl;
    show_usage_S (argv[0]);
    cout << endl;
    cout << endl;
  }
  
  // ---------------------------
  if (!((argc < 2) ? read_metafile_S() : read_metafile_S (argv[1])))
  {
    cout << endl;
    FATAL_MSG	("Unable to select data from metafile");
    cout << endl;
    show_metafile_structure_S ();
    show_metafile_sample_S ();
    cout << endl;
    return ;
  }

  cout << endl;

size_t	number_of_tapes;

  for (size_t i = 0; i < metafile_data_s.size(); i++)
  {
     vector<string>	input_words_files_names;

     number_of_tapes = atoi (metafile_data_s[i][METAFILE_FIELD__NUMBER_OF_TAPES].c_str());

     for (size_t j = METAFILE_FIELD__INPUT_WORDS_FILE1_NAME; 
		 j < metafile_data_s[i].size(); 
		 j++
		 )
     {
       input_words_files_names.push_back (metafile_data_s[i][j]);
     }

     //copy (metafile_data_s[i].begin() + METAFILE_FIELD__INPUT_WORDS_FILE1_NAME, metafile_data_s[i].end(), input_words_files_names.begin());

     //assert (number_of_tapes != 0);
     invoke_post_machine_S (
	(i + 1),
	number_of_tapes,
	metafile_data_s[i][METAFILE_FIELD__DESCR_FILE_NAME], 
	metafile_data_s[i][METAFILE_FIELD__COMMANDS_FILE_NAME], 
	input_words_files_names
	);

  }

} // run



