// ==============================================================
//
//  Copyright (c) 2002-2003 Alex Vinokur.
//
//  ------------------------------------------------------------
//  This file is part of C++ Simulator of a Post Machine.
//
//  C++ Simulator of a Post Machine is free software; 
//  you can redistribute it and/or modify it
//  under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License,
//  or (at your option) any later version.
//
//  C++ Simulator of a Post Machine is distributed in the hope 
//  that it will be useful, but WITHOUT ANY WARRANTY; 
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with C++ Simulator of a Post Machine; 
//  if not, write to the Free Software Foundation, Inc., 
//  59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//  ------------------------------------------------------------
// 
//  mailto:alexvn@connect.to
//  http://up.to/alexv
//
// ==============================================================




// ##############################################################
//
//  SOFTWARE : Nondeterministic Post Machine (C++ Simulator)
//  FILE     : run.h
//
//  DESCRIPTION :
//         Class Run (Definition)
//
// ##############################################################



// ###############
#ifndef _RUN_H
#define _RUN_H

#include "post-m.h"

#define	DEFAULT_NUMBER_OF_TAPES		1
#define	DEFAULT_METAFILE		"metafile"
#define	DEFAULT_DESCR_FILE		"descr"
#define	DEFAULT_COMMANDS_FILE		"commands"
#define	DEFAULT_INPUT_WORDS_FILE	"iwords"

// =========
class Aux
{
friend class Run;
  private :
    vector<string>	help_request_keys_;
    void		init ();

  public :
    Aux ();
    ~Aux ();
};


// =========
class Run
{
  private :
    // ------ Private Data ------
    size_t		number_of_tapes_;
    bool		check_results_;
    size_t		serial_number_;

    static string	metafile_name_s;
    string		descr_file_name_;
    string		commands_file_name_;
    vector<string>	input_words_files_names_;

    static vector<vector<string> >	metafile_data_s;
    vector<vector<string> >		descr_file_data_;
    vector<vector<string> >		commands_file_data_;
    vector<vector<vector<string> > >	input_words_files_data_;
    vector<command_t>	 		initial_command_numbers_;
    vector<command_t>			halting_command_numbers_;
    vector<command_t>	   		internal_command_numbers_;
    Commands_t	   			commands_map_;
    command_t				last_command_no_;

    static const vector<string>		help_request_keys_s;

    // ------ Private Methods ------
    bool	init_number_of_tapes (size_t number_of_tapes_i);
    bool	init_descr (const string& descr_file_name_i);
    bool	init_commands (const string& commands_file_name_i);
    bool	parse_command (
			size_t                  line_no_i,
			const vector<string>&   cmd_i,
			command_t&              cur_command_no_o,
			vector<OperationType>&  cur_operations_o,
			vector<command_t>&      next_command_no_o,
			bool&			cur_cmd_is_initial_o,
			bool&			cur_cmd_is_halting_o,
			bool&			cur_cmd_is_internal_o
			);
    bool	create_commands ();
    bool	init_input_words (const vector<string>& input_words_files_names_i);


    static bool	read_file_S (const string& filename_i, vector<vector<string> >& data_o);
    static bool	read_metafile_S (const string& filename_i = DEFAULT_METAFILE);

    static void	invoke_post_machine_S (
		size_t			serial_number_i,
		size_t			total_tapes_i,
		const string&		descr_file_name_i,
		const string&		commands_file_name_i,
		const vector<string>&	input_words_files_names_i
		);

    void	invoke_post_machine (
		const vector<string>&		descr_file_data_i,
		const vector<string>&		commands_file_data_i,
		const vector<vector<string> >&	input_words_files_data_i
		);
    
    bool	read_descr_file ();
    bool	read_commands_file ();
    bool	read_input_words_file (const string& filename_i);

    static void	show_metafile_structure_S ();
    static void	show_descr_file_structure_S ();
    static void	show_commands_file_structure_S ();
    void	show_commands_file_structure () const;
    static void	show_input_words_file_structure_S ();

    static void	show_metafile_sample_S ();
    static void	show_descr_file_sample_S ();
    static void	show_commands_file_sample_S ();
    static void	show_input_words_file_sample_S ();
							
    static void	show_metafile_content_S ();
    void	show_descr_file_content () const;
    void	show_commands_file_content () const;
    void	show_input_words_files_content () const;

    static void	show_help_S(const string& exe_name_i);
    static void	show_sample_S();
    static void	show_usage_S(const string& exe_name_i);

    bool	invoke_machine();

  public :
    Run (
		size_t			serial_number_i,
		size_t			total_tapes_i,
		const string&		descr_file_name_i,
		const string&		commands_file_name_i,
		const vector<string>&	input_words_files_names_i
		);
    ~Run ();  
    static void	run (uint argc, char** argv);
};

#endif	// _RUN_H
